/********************************************************************************************************
 * @file	app_ndma.c
 *
 * @brief	This is the source file for TLSR8231
 *
 * @author	Driver Group
 * @date	May 12, 2019
 *
 * @par     Copyright (c) 2018, Telink Semiconductor (Shanghai) Co., Ltd. ("TELINK")
 *          All rights reserved.
 *
 *          Redistribution and use in source and binary forms, with or without
 *          modification, are permitted provided that the following conditions are met:
 *
 *              1. Redistributions of source code must retain the above copyright
 *              notice, this list of conditions and the following disclaimer.
 *
 *              2. Unless for usage inside a TELINK integrated circuit, redistributions
 *              in binary form must reproduce the above copyright notice, this list of
 *              conditions and the following disclaimer in the documentation and/or other
 *              materials provided with the distribution.
 *
 *              3. Neither the name of TELINK, nor the names of its contributors may be
 *              used to endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              4. This software, with or without modification, must only be used with a
 *              TELINK integrated circuit. All other usages are subject to written permission
 *              from TELINK and different commercial license may apply.
 *
 *              5. Licensee shall be solely responsible for any claim to the extent arising out of or
 *              relating to such deletion(s), modification(s) or alteration(s).
 *
 *          THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *          ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *          WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *          DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDER BE LIABLE FOR ANY
 *          DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *          (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *          LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *          ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *          (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *          SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************************************/
#include "app_config.h"

#if (UART_MODE==UART_NDMA)


#define REC_BUFF_LEN    16
#define TRANS_BUFF_LEN  16

volatile unsigned char uart_rx_flag;
volatile unsigned int  uart_ndmairq_cnt;
volatile unsigned char uart_ndmairq_index;
volatile unsigned char uart_cts_count=0;
__attribute__((aligned(4))) unsigned char rec_buff[REC_BUFF_LEN]={0};
__attribute__((aligned(4))) unsigned char trans_buff[TRANS_BUFF_LEN] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88,
																		0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
void user_init()
{
	delay_ms(2000);  //leave enough time for SWS_reset when power on

	uart_set_pin(UART_TX_PB4, UART_RX_PB5);// uart tx/rx pin set

	uart_reset();  //will reset uart digital registers from 0x90 ~ 0x9f, so uart setting must set after this reset

	//baud rate: 9600
	uart_init_baudrate(115200,CLOCK_SYS_CLOCK_HZ,(unsigned short *)&rec_buff,sizeof(rec_buff));

	uart_Parity(PARITY_NONE);

	uart_init_StopBit(STOP_BIT_ONE);

#if( FLOW_CTR == NORMAL)

	uart_dma_en(0, 0);

	irq_clr_mask(FLD_IRQ_DMA_EN);

	dma_set_irq_en(FLD_DMA_CHN_UART_RX | FLD_DMA_CHN_UART_TX, 0);

	uart_irq_en(1,0);   //uart RX irq enable

	uart_ndma_set_triglevel(1,0);	//set the trig level. 1 indicate one byte will occur interrupt

#elif( FLOW_CTR ==  USE_CTS )
	//CTS pin.It can be C2.
	uart_set_cts(1, STOP_VOLT, UART_CTS_PC2);

	uart_dma_en(0, 0);				       //Disable DMA

	irq_clr_mask(FLD_IRQ_DMA_EN);

	dma_set_irq_en(FLD_DMA_CHN_UART_RX | FLD_DMA_CHN_UART_TX, 0);	//Disable DMA irq

	uart_irq_en(0,0);   //uart RX irq disable



#elif( FLOW_CTR ==   USE_RTS )
	// RTS pin : GPIO_PC3 GPIO_PD1
	uart_set_rts(1, RTS_MODE, RTS_THRESH, RTS_INVERT,UART_RTS_PC3);
	uart_dma_en(0, 0);

	irq_clr_mask(FLD_IRQ_DMA_EN);

	dma_set_irq_en(FLD_DMA_CHN_UART_RX | FLD_DMA_CHN_UART_TX, 0);

	uart_irq_en(1,0);   //uart RX irq enable

	uart_ndma_set_triglevel(RTS_THRESH,0);


#endif
}

/////////////////////////////////////////////////////////////////////
// main loop flow
/////////////////////////////////////////////////////////////////////
void main_loop (void)
{
	delay_ms(1000);
#if( FLOW_CTR == NORMAL)

	for(unsigned char i=0;i<TRANS_BUFF_LEN;i++){
		uart_ndma_send_byte(trans_buff[i]);
	}
	if(uart_rx_flag>0){
		uart_ndmairq_cnt=0; //Clear uart_ndmairq_cnt
		uart_rx_flag=0;
		for(unsigned char i=0;i<TRANS_BUFF_LEN;i++){
			uart_ndma_send_byte(rec_buff[i]);
		}
	}
#elif( FLOW_CTR ==  USE_CTS )
	uart_ndma_send_byte(trans_buff[uart_cts_count]);
	uart_cts_count++;
	if(uart_cts_count == 16)
	{
		uart_cts_count=0;
	}

#elif( FLOW_CTR ==  USE_RTS )

	while(1);
#endif


}





#endif















